/****************************************************************************
**
** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the Qt3D module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

uniform sampler2D qt_Texture0;
uniform sampler2D qt_Texture1;
varying mediump vec4 lightingColor;
varying highp vec4 texCoord0;
uniform mediump float glowFactor;

void main(void)
{
    vec4 unlitColor0 = texture2D(qt_Texture0, texCoord0.st);
    vec4 unlitColor1 = texture2D(qt_Texture1, texCoord0.st);

    // interpolate between the two textures
    vec4 unlitColor = mix(unlitColor0, unlitColor1, glowFactor);

    // Apply lighting
    vec4 litColor = lightingColor * unlitColor;
    // Normally, we'd just paint this lit colour like so:
//    gl_FragColor = litColor;

    // However, we want a cool glow effect, because the sun is generating
    // it's own light.

    // Multiplying the color by itself will washing out the darker colours,
    // leaving the brighter colours brighter by comparison
    vec4 powerColor = unlitColor * unlitColor;
    powerColor = powerColor * powerColor ;

    // Then combine the higher contrast color and the lit color
    gl_FragColor= mix(litColor, powerColor, glowFactor);
}
